; 29.11.2001 - SYSEECHO.SYS - shows cmdline arg on DOS console
; plus translates $ to ESC (FreeDOS 2035+ has working kernel built-in EECHO)
; 1.10.2004 DELAY version waits 1 second at each (not displayed) $ sign.
; %define DELAY 1	; define to make $ cause DELAY instead of print ESC

%define NEWSTYLE 1	; *both* delay and escape, new syntax (2/2005)
			; remove to compile classic syseecho or sysdelay
; Feature: Show argument as string, translating $ to ESC and ~ to a
; delay (1 second, or if scroll lock pressed or active, none or until-key)
; and $$ to clear screen. Compile: nasm -o eecho2.sys syseecho.asm

%ifdef NEWSTYLE
%define DELAY 1
%define ESCAPE 1
%endif
%ifndef DELAY
%define ESCAPE 1	; if not delay, then ESCAPE
%endif
%ifndef ESCAPE
%define DELAY 1		; if not escape, then DELAY
%endif

; Classic syseecho: nasm -o syseecho.sys syseecho.asm
; Feature: Show argument as string, translating $ to ESC char.
;          The string $[2J triggers a clear screen / home even
;          if no ANSI is loaded (new 2/2005).
; Notice that new FreeDOS kernels have built-in EECHO which works
; like syseecho. Older versions only EECHO to TTY, not to console.

; Classic sysdelay: nasm -DDELAY=1 -o sysdelay.sys syseecho.asm
; Feature: Show argument as string, waiting at each $ char.
;          Default is to wait 1 second each time, but if you have
;          scroll lock on, it will wait for a key press each time.
;          Keep the scroll lock key pressed to bypass all delays.

        org  0  ; SYS file

next    dw -1,-1
attrib  dw 1000000000000100b    ; very nully NUL device
stra    dw strat	; *offset* prepare, store ES:BX (far call)
inr     dw intr  	; *offset* do the real work (far call)
%ifdef NEWSTYLE
	db 'ECHOWAIT'	; not important...
%else
%ifdef DELAY
nam	db 'DELAY$$$'	; not important, does not stay in RAM anyway
%else
nam     db 'EECHO$$$'   ; driver name will become a special filename,
%endif			; so better not use a too normal name...
%endif
pb      dd 12345678h

strat:  mov word [cs:pb+2],es
        mov word [cs:pb],bx	; we could copy the contents of the
        RETF			; referenced data structure here, too

intr:   pushf
	push ax
        push bx
        push es
        les bx,[cs:pb]
        mov byte [es:bx+17h],0		; no error
        and word [es:bx+3],7FFDh	; not busy / no error
        or word [es:bx+3],1		; ack state
        cmp byte [es:bx+2],0		; INIT ?
	jz inst
nix:    pop es
        pop bx
	pop ax
        popf
        RETF


inst:   mov word [es:bx+10h],cs	; eof tsr
        mov word [es:bx+0Eh],0	; eof tsr (thus no tsr)

	; we could check for a minimum DOS version here (4.0 ?)

	push bx
	mov ax,0e0dh
	mov bx,7	; TTY ignores color in text modes!
	int 10h		; CR (TTY)
	mov ax,0e0ah
	mov bx,7
	int 10h		; LF (TTY)
	pop bx

	les bx,[es:bx+12h]	; pointer to command line

skp:	mov al,[es:bx]
	cmp al,' '
	jb hlp		; command line ended with name, no args
	jz rpt		; the real args
	inc bx		; skip filename of sys
	jmp short skp

rpt:	mov ax,[es:bx]	; char plus lookahead
	cmp al,0
	jz enst
	cmp al,10
	jz enst
	cmp al,13
	jz enst
	cmp al,'$'	; $ -> delay or ESC (newstyle: ESC)
	jz xesc
%ifdef NEWSTYLE
	cmp al,'~'	; newstyle: ~ -> delay
	jz xdelay
%endif
	cmp al,'A'
	jb norm
	cmp al,'Z'
	ja norm
%ifndef ESCAPE
	or al,32	; downcase, because FreeDOS upcases. sigh...
	; NOT good in conjunction with ESC sequences - %if added 2/2005.
%endif
	jmp short norm

%ifdef NEWSTYLE
xdelay:
%else
xesc:
%endif

%ifdef DELAY		; $ -> wait 1 second (newstyle: ~ -> ...)
	call wait1	; wait one second
	jmp short cdone
%endif

%ifdef ESCAPE
xesc:	mov al,27	; $ -> ESC
%ifndef NEWSTYLE
        cmp ah,'['	; ANSI upcoming?
	jnz norm	; no interesting ESC code
	cmp word [es:bx+2],'2J'	; CLS one?
	jnz norm
	push ax
	call cls	; clear screen without ANSI help...
	pop ax		; ... but let ANSI do it again, if present.
%else
	cmp ah,'$'	; newstyle: $$ -> CLS
	jnz norm
	call cls
	inc bx		; skip over 2nd $
	jmp short cdone
%endif
%endif

norm:	mov ah,2
	push dx
	mov dl,al	; char
	int 21h		; CHAR (dossy output!)
	pop dx
cdone:	inc bx		; next char
	jmp short rpt

enst:	mov ax,0e0dh
	mov bx,7
	int 10h		; CR (TTY)
	mov ax,0e0ah
	mov bx,7
	int 10h		; LF (TTY)

	jmp nix		; DONE
		
hlp:	mov bx,cs
	mov es,bx
	mov bx,hlpmsg	; *offset* help message
	jmp short rpt	; show string

%ifdef NEWSTYLE	; new help screen
hlpmsg	db 'Public Domain EECHO2.SYS by EA 2005:'
	db ' show arguments on screen.',13,10
	db 9,'dollar is shown as ESC, dollar dollar causes CLS',13,10
	db 9,'tilde means 1 sec. wait (scroll lock pressed/active'
	db ' overrides).',13,10,0
%else
%ifdef DELAY	; help shown if no arguments given
hlpmsg	db 'sysdelay.sys: shows command line and waits 1 second at each'
	db 13,10
	db 'dollar sign. Dollar signs are not shown. Public Domain by EA.'
	db 13,10,0
%else
hlpmsg	db 'syseecho.sys: shows command line, translating dollar signs to'
	db ' esc. By EA.',0
%endif
%endif

%ifdef ESCAPE
cls:	push ax
	push bx
	push cx
	;
	push ds
	mov ax,40h	; BIOS data segment
	mov ds,ax
	mov cl,[ds:84h]	; YMAX, on EGA/newer
	cmp cl,19
	jae sanerows
	mov cl,25-1	; if no plausible value, assume 25 rows
sanerows:
	pop ds
	;
	push dx
	mov ah,0fh	; get mode and page
	int 10h		; returns page BH, columns AH (and mode AL)
	dec ah
	mov ch,ah	; XMAX
	xchg cl,ch
	push cx		; lower right corner
	mov ah,7	; if no color text mode, use fixed color 7
	cmp al,3
	ja fixcolor
	;
	mov ah,2
	mov dl,'-'	; test char
	int 21h		; DOS output in current color
	mov ax,0e08h
	int 10h		; backspace, page BH, (any) color BL
	mov ah,0
	int 10h		; read char / color at cursor position on page BH
fixcolor:
	pop dx		; lower right corner
	push bx		; bh: page
	mov bh,ah	; color (ignore char AL)
	mov ax,0600h	; "scroll up box by 0 lines" (clear)
	xor cx,cx	; start at upper left corner
	push bp		; bios can destroy bp
	int 10h
	pop bp
	pop bx
	xor dx,dx	; upper left corner
	mov ah,2
	int 10h		; set cursor position on page bh
	pop dx
	pop cx
	pop bx
	pop ax
	ret
%endif

%ifdef DELAY
wait1:	push ds			; wait a second
	mov ax,40h		; BIOS data segment
	mov ds,ax
	mov ax,[ds:17h]		; some shift states
	test ah,10h		; scroll lock PRESSED?
	jnz skipwait
	test al,10h		; scroll lock state ACTIVE?
	jnz keywait
	mov al,[ds:6ch]		; timer tick count
	add al,18		; 1 second is 18.2 ticks
	mov ah,al
dloop:	mov al,[ds:6ch]
	cmp al,ah
	; hlt			; only do this if interrupts are
				; enabled and HLT has no side effects
	jnz dloop		; not yet elapsed? then keep waiting.
skipwait:
	pop ds
	ret

keywait:
	xor ax,ax		; wait for a key instead of delaying
	int 16h
	jmp short skipwait
%endif

